"""Babamul topics."""

from typing import Literal

ALL_TOPICS = [
    "babamul.ztf.no-lsst-match.stellar",
    "babamul.ztf.no-lsst-match.hosted",
    "babamul.ztf.no-lsst-match.hostless",
    "babamul.ztf.lsst-match.stellar",
    "babamul.ztf.lsst-match.hosted",
    "babamul.ztf.lsst-match.hostless",
    "babamul.lsst.no-ztf-match.stellar",
    "babamul.lsst.no-ztf-match.hosted",
    "babamul.lsst.no-ztf-match.hostless",
    "babamul.lsst.no-ztf-match.unknown",
    "babamul.lsst.ztf-match.stellar",
    "babamul.lsst.ztf-match.hosted",
    "babamul.lsst.ztf-match.hostless",
    "babamul.lsst.ztf-match.unknown",
]

TopicType = Literal[
    "babamul.ztf.no-lsst-match.stellar",
    "babamul.ztf.no-lsst-match.hosted",
    "babamul.ztf.no-lsst-match.hostless",
    "babamul.ztf.lsst-match.stellar",
    "babamul.ztf.lsst-match.hosted",
    "babamul.ztf.lsst-match.hostless",
    "babamul.lsst.no-ztf-match.stellar",
    "babamul.lsst.no-ztf-match.hosted",
    "babamul.lsst.no-ztf-match.hostless",
    "babamul.lsst.no-ztf-match.unknown",
    "babamul.lsst.ztf-match.stellar",
    "babamul.lsst.ztf-match.hosted",
    "babamul.lsst.ztf-match.hostless",
    "babamul.lsst.ztf-match.unknown",
]
